@@ -6,7 +6,7 @@ import time
 import sys
 
 
-__version__ = '0.2'
+__version__ = '0.3'
 
 
 logger = logging.getLogger('usagestats')
@@ -24,7 +24,7 @@ class Prompt(object):
                 raise TypeError("Expects either 'prompt' or both 'enable' and "
                                 "'disable'")
             self.prompt = (
-                    "Uploading usage statistics is currently {{status}}\n"
+                    "Uploading usage statistics is currently disabled\n"
                     "Please help us by providing anonymous usage statistics; "
                     "you can enable this\nby running:\n"
                     "    {enable}\n"
@@ -52,38 +52,65 @@ def SESSION_TIME(stats, info):
 
 
 def PYTHON_VERSION(stats, info):
-    python = ';'.join([str(c) for c in sys.version_info] + [sys.version])
+    # Some versions of Python have a \n in sys.version!
+    version = sys.version.replace(' \n', ' ').replace('\n', ' ')
+    python = ';'.join([str(c) for c in sys.version_info] + [version])
     info.append(('python', python))
 
 
 def _encode(s):
-    if isinstance(s, bytes):
-        return s
-    if str == bytes:  # Python 2
-        if isinstance(s, unicode):
-            return s.encode('utf-8')
-        else:
-            return str(s)
-    else:  # Python 3
-        if isinstance(s, str):
-            return s.encode('utf-8')
-        else:
-            return str(s).encode('utf-8')
+    if not isinstance(s, bytes):
+        if str == bytes:  # Python 2
+            if isinstance(s, unicode):
+                s = s.encode('utf-8')
+            else:
+                s = str(s)
+        else:  # Python 3
+            if isinstance(s, str):
+                s = s.encode('utf-8')
+            else:
+                s = str(s).encode('utf-8')
+    if b'\n' in s:
+        s = s.replace(b'\n', b' ')
+    return s
 
 
 class Stats(object):
-    DISABLED, ENABLED, UNSET = range(3)
+    ERRORED, DISABLED_ENV, DISABLED, UNSET, ENABLED = range(5)
+
+    @property
+    def enabled(self):
+        return self.status in (Stats.UNSET, Stats.ENABLED)
+
+    @property
+    def enableable(self):
+        return self.status is not Stats.ERRORED
+
+    @property
+    def disableable(self):
+        return self.status is not Stats.ERRORED
+
+    @property
+    def recording(self):
+        return self.status in (Stats.UNSET, Stats.ENABLED)
+
+    @property
+    def sending(self):
+        return self.status is Stats.ENABLED
 
     def __init__(self, location, prompt, drop_point,
                  version, unique_user_id=False,
-                 env_var='PYTHON_USAGE_STATS'):
+                 env_var='PYTHON_USAGE_STATS',
+                 ssl_verify=None):
         self.started_time = time.time()
 
-        env_var = os.environ.get(env_var).lower()
+        self.ssl_verify = ssl_verify
+
+        env_var = os.environ.get(env_var, '').lower()
         if env_var not in (None, '', '1', 'on', 'enabled', 'yes', 'true'):
-            self.enabled = Stats.DISABLED
+            self.status = Stats.DISABLED_ENV
         else:
-            self.enabled = Stats.UNSET
+            self.status = Stats.UNSET
         self.location = os.path.expanduser(location)
         self.drop_point = drop_point
         self.version = version
@@ -96,22 +123,22 @@ class Stats(object):
             raise TypeError("'prompt' should either be None, a Prompt or a "
                             "string")
 
-        if not os.path.isdir(self.location):
+        if self.enabled and not os.path.isdir(self.location):
             try:
                 os.makedirs(self.location, 0o700)
             except OSError:
                 logger.warning("Couldn't create %s, usage statistics won't be "
                                "collected", self.location)
-                self.enabled = Stats.DISABLED
+                self.status = Stats.ERRORED
 
         status_file = os.path.join(self.location, 'status')
         if self.enabled and os.path.exists(status_file):
             with open(status_file, 'r') as fp:
                 status = fp.read().strip()
             if status == 'ENABLED':
-                self.enabled = Stats.ENABLED
+                self.status = Stats.ENABLED
             elif status == 'DISABLED':
-                self.enabled = Stats.DISABLED
+                self.status = Stats.DISABLED
 
         if self.enabled and unique_user_id:
             user_id_file = os.path.join(self.location, 'user_id')
@@ -128,25 +155,32 @@ class Stats(object):
 
         self.notes = []
 
+        self.note([('version', self.version)])
+
     def enable_reporting(self):
-        if self.enabled is Stats.DISABLED:
+        if not self.enableable:
             logger.critical("Can't enable reporting")
             return
-        elif self.enabled is not Stats.ENABLED:
-            self.enabled = Stats.ENABLED
-            status_file = os.path.join(self.location, 'status')
-            with open(status_file, 'w') as fp:
-                fp.write('ENABLED')
+        self.status = Stats.ENABLED
+        status_file = os.path.join(self.location, 'status')
+        with open(status_file, 'w') as fp:
+            fp.write('ENABLED')
 
     def disable_reporting(self):
-        if self.enabled is Stats.DISABLED:
+        if not self.disableable:
             logger.critical("Can't disable reporting")
             return
-        else:
-            self.enabled = Stats.DISABLED
-            status_file = os.path.join(self.location, 'status')
-            with open(status_file, 'w') as fp:
-                fp.write('DISABLED')
+        self.status = Stats.DISABLED
+        status_file = os.path.join(self.location, 'status')
+        with open(status_file, 'w') as fp:
+            fp.write('DISABLED')
+        if os.path.exists(self.location):
+            old_reports = [f for f in os.listdir(self.location)
+                           if f.startswith('report_')]
+            for old_filename in old_reports:
+                fullname = os.path.join(self.location, old_filename)
+                os.remove(fullname)
+            logger.info("Deleted %d pending reports", len(old_reports))
 
     @staticmethod
     def _to_notes(info):
@@ -158,11 +192,11 @@ class Stats(object):
             return info
 
     def note(self, info):
-        if self.enabled:
+        if self.recording:
             self.notes.extend(self._to_notes(info))
 
     def submit(self, info, *flags):
-        if not self.enabled:
+        if not self.recording:
             return
 
         all_info, self.notes = self.notes, None
@@ -178,23 +212,39 @@ class Stats(object):
         if self.user_id:
             all_info.insert(1, ('user', self.user_id))
 
-        logger.info("Generated report:\n%r" % (all_info,))
+        logger.info("Generated report:\n%r", (all_info,))
 
         # Current report
-        filename = 'report_%d_%d.txt' % (secs, msecs)
         def generator():
             for key, value in all_info:
                 yield _encode(key) + b':' + _encode(value) + b'\n'
+        filename = 'report_%d_%d.txt' % (secs, msecs)
+
+        # Save current report and exit, unless user has opted in
+        if not self.sending:
+            fullname = os.path.join(self.location, filename)
+            with open(fullname, 'wb') as fp:
+                for l in generator():
+                    fp.write(l)
+
+            # Show prompt
+            sys.stderr.write(self.prompt.prompt)
+            return
 
         # Post previous reports
         old_reports = [f for f in os.listdir(self.location)
                        if f.startswith('report_')]
         old_reports.sort()
+        old_reports = old_reports[:4]  # Only upload 5 at a time
         for old_filename in old_reports:
             fullname = os.path.join(self.location, old_filename)
             try:
                 with open(fullname, 'rb') as fp:
-                    requests.post(self.drop_point, data=fp)
+                    # FIXME: ``data=generator()`` would make requests stream,
+                    # which is currently not a good idea (WSGI chokes on it)
+                    r = requests.post(self.drop_point, data=fp.read(),
+                                      timeout=1, verify=self.ssl_verify)
+                    r.raise_for_status()
             except Exception as e:
                 logger.warning("Couldn't upload %s: %s", old_filename, str(e))
                 break
@@ -204,8 +254,10 @@ class Stats(object):
 
         # Post current report
         try:
-            r = requests.post(self.drop_point, data=generator())
-            r.raise_for_status()
+            # FIXME: ``data=generator()`` would make requests stream, which is
+            # currently not a good idea (WSGI chokes on it)
+            r = requests.post(self.drop_point, data=b''.join(generator()),
+                              timeout=1, verify=self.ssl_verify)
         except requests.RequestException as e:
             logger.warning("Couldn't upload report: %s", str(e))
             fullname = os.path.join(self.location, filename)
@@ -213,8 +265,8 @@ class Stats(object):
                 for l in generator():
                     fp.write(l)
         else:
-            logger.info("Submitted current report")
-
-        # Show prompt
-        if self.enabled is Stats.UNSET:
-            sys.stderr.write(self.prompt)
+            try:
+                r.raise_for_status()
+                logger.info("Submitted current report")
+            except requests.RequestException as e:
+                logger.warning("Server rejected report: %s", str(e))
